@@ -4,10 +4,12 @@ from typing import AsyncIterator, List, Tuple, Union
 import h11
 
 from .._backends.auto import AsyncSocketStream
-from .._exceptions import ProtocolError, map_exceptions
+from .._bytestreams import PlainByteStream, AsyncIteratorByteStream
+from .._exceptions import RemoteProtocolError, LocalProtocolError, map_exceptions
 from .._types import URL, Headers, TimeoutDict
 from .._utils import get_logger
-from .base import AsyncByteStream, AsyncHTTPTransport, ConnectionState
+from .base import AsyncByteStream, ConnectionState
+from .http import AsyncBaseHTTPConnection
 
 H11Event = Union[
     h11.Request,
@@ -21,8 +23,8 @@ H11Event = Union[
 logger = get_logger(__name__)
 
 
-class AsyncHTTP11Connection(AsyncHTTPTransport):
-    READ_NUM_BYTES = 4096
+class AsyncHTTP11Connection(AsyncBaseHTTPConnection):
+    READ_NUM_BYTES = 64 * 1024
 
     def __init__(
         self, socket: AsyncSocketStream, ssl_context: SSLContext = None,
@@ -34,6 +36,15 @@ class AsyncHTTP11Connection(AsyncHTTPTransport):
 
         self.state = ConnectionState.ACTIVE
 
+    def __repr__(self) -> str:
+        return f"<AsyncHTTP11Connection state={self.state}>"
+
+    def info(self) -> str:
+        return f"HTTP/1.1, {self.state.name}"
+
+    def get_state(self) -> ConnectionState:
+        return self.state
+
     def mark_as_ready(self) -> None:
         if self.state == ConnectionState.IDLE:
             self.state = ConnectionState.READY
@@ -47,7 +58,7 @@ class AsyncHTTP11Connection(AsyncHTTPTransport):
         timeout: TimeoutDict = None,
     ) -> Tuple[bytes, int, bytes, List[Tuple[bytes, bytes]], AsyncByteStream]:
         headers = [] if headers is None else headers
-        stream = AsyncByteStream() if stream is None else stream
+        stream = PlainByteStream(b"") if stream is None else stream
         timeout = {} if timeout is None else timeout
 
         self.state = ConnectionState.ACTIVE
@@ -60,15 +71,18 @@ class AsyncHTTP11Connection(AsyncHTTPTransport):
             reason_phrase,
             headers,
         ) = await self._receive_response(timeout)
-        stream = AsyncByteStream(
+        response_stream = AsyncIteratorByteStream(
             aiterator=self._receive_response_data(timeout),
             aclose_func=self._response_closed,
         )
-        return (http_version, status_code, reason_phrase, headers, stream)
+        return (http_version, status_code, reason_phrase, headers, response_stream)
 
-    async def start_tls(self, hostname: bytes, timeout: TimeoutDict = None) -> None:
+    async def start_tls(
+        self, hostname: bytes, timeout: TimeoutDict = None
+    ) -> AsyncSocketStream:
         timeout = {} if timeout is None else timeout
         self.socket = await self.socket.start_tls(hostname, self.ssl_context, timeout)
+        return self.socket
 
     async def _send_request(
         self, method: bytes, url: URL, headers: Headers, timeout: TimeoutDict,
@@ -78,7 +92,8 @@ class AsyncHTTP11Connection(AsyncHTTPTransport):
         """
         logger.trace("send_request method=%r url=%r headers=%s", method, url, headers)
         _scheme, _host, _port, target = url
-        event = h11.Request(method=method, target=target, headers=headers)
+        with map_exceptions({h11.LocalProtocolError: LocalProtocolError}):
+            event = h11.Request(method=method, target=target, headers=headers)
         await self._send_event(event, timeout)
 
     async def _send_request_body(
@@ -138,11 +153,7 @@ class AsyncHTTP11Connection(AsyncHTTPTransport):
         Read a single `h11` event, reading more data from the network if needed.
         """
         while True:
-            with map_exceptions({h11.RemoteProtocolError: ProtocolError}):
-                logger.debug(
-                    "h11.RemoteProtocolError exception their_state=%r",
-                    self.h11_state.their_state,
-                )
+            with map_exceptions({h11.RemoteProtocolError: RemoteProtocolError}):
                 event = self.h11_state.next_event()
 
             if event is h11.NEED_DATA:
