@@ -8,16 +8,13 @@ from h2.config import H2Configuration
 from h2.exceptions import NoAvailableStreamIDError
 from h2.settings import SettingCodes, Settings
 
-from .._backends.auto import AsyncLock, AsyncSocketStream, AutoBackend
-from .._exceptions import ProtocolError
+from .._backends.auto import AsyncLock, AsyncSemaphore, AsyncSocketStream, AutoBackend
+from .._bytestreams import PlainByteStream, AsyncIteratorByteStream
+from .._exceptions import PoolTimeout, RemoteProtocolError
 from .._types import URL, Headers, TimeoutDict
 from .._utils import get_logger
-from .base import (
-    AsyncByteStream,
-    AsyncHTTPTransport,
-    ConnectionState,
-    NewConnectionRequired,
-)
+from .base import AsyncByteStream, ConnectionState, NewConnectionRequired
+from .http import AsyncBaseHTTPConnection
 
 logger = get_logger(__name__)
 
@@ -29,8 +26,8 @@ def get_reason_phrase(status_code: int) -> bytes:
         return b""
 
 
-class AsyncHTTP2Connection(AsyncHTTPTransport):
-    READ_NUM_BYTES = 4096
+class AsyncHTTP2Connection(AsyncBaseHTTPConnection):
+    READ_NUM_BYTES = 64 * 1024
     CONFIG = H2Configuration(validate_inbound_headers=False)
 
     def __init__(
@@ -51,6 +48,12 @@ class AsyncHTTP2Connection(AsyncHTTPTransport):
 
         self.state = ConnectionState.ACTIVE
 
+    def __repr__(self) -> str:
+        return f"<AsyncHTTP2Connection state={self.state}>"
+
+    def info(self) -> str:
+        return f"HTTP/2, {self.state.name}, {len(self.streams)} streams"
+
     @property
     def init_lock(self) -> AsyncLock:
         # We do this lazily, to make sure backend autodetection always
@@ -67,8 +70,24 @@ class AsyncHTTP2Connection(AsyncHTTPTransport):
             self._read_lock = self.backend.create_lock()
         return self._read_lock
 
-    async def start_tls(self, hostname: bytes, timeout: TimeoutDict = None) -> None:
-        pass
+    @property
+    def max_streams_semaphore(self) -> AsyncSemaphore:
+        # We do this lazily, to make sure backend autodetection always
+        # runs within an async context.
+        if not hasattr(self, "_max_streams_semaphore"):
+            max_streams = self.h2_state.local_settings.max_concurrent_streams
+            self._max_streams_semaphore = self.backend.create_semaphore(
+                max_streams, exc_class=PoolTimeout
+            )
+        return self._max_streams_semaphore
+
+    async def start_tls(
+        self, hostname: bytes, timeout: TimeoutDict = None
+    ) -> AsyncSocketStream:
+        raise NotImplementedError("TLS upgrade not supported on HTTP/2 connections.")
+
+    def get_state(self) -> ConnectionState:
+        return self.state
 
     def mark_as_ready(self) -> None:
         if self.state == ConnectionState.IDLE:
@@ -91,6 +110,8 @@ class AsyncHTTP2Connection(AsyncHTTPTransport):
                 await self.send_connection_init(timeout)
                 self.sent_connection_init = True
 
+        await self.max_streams_semaphore.acquire()
+        try:
             try:
                 stream_id = self.h2_state.get_next_available_stream_id()
             except NoAvailableStreamIDError:
@@ -99,10 +120,13 @@ class AsyncHTTP2Connection(AsyncHTTPTransport):
             else:
                 self.state = ConnectionState.ACTIVE
 
-        h2_stream = AsyncHTTP2Stream(stream_id=stream_id, connection=self)
-        self.streams[stream_id] = h2_stream
-        self.events[stream_id] = []
-        return await h2_stream.request(method, url, headers, stream, timeout)
+            h2_stream = AsyncHTTP2Stream(stream_id=stream_id, connection=self)
+            self.streams[stream_id] = h2_stream
+            self.events[stream_id] = []
+            return await h2_stream.request(method, url, headers, stream, timeout)
+        except Exception:
+            self.max_streams_semaphore.release()
+            raise
 
     async def send_connection_init(self, timeout: TimeoutDict) -> None:
         """
@@ -192,7 +216,7 @@ class AsyncHTTP2Connection(AsyncHTTPTransport):
             logger.trace("receive_event stream_id=%r event=%s", event_stream_id, event)
 
             if hasattr(event, "error_code"):
-                raise ProtocolError(event)
+                raise RemoteProtocolError(event)
 
             if event_stream_id in self.events:
                 self.events[event_stream_id].append(event)
@@ -231,15 +255,18 @@ class AsyncHTTP2Connection(AsyncHTTPTransport):
         await self.socket.write(data_to_send, timeout)
 
     async def close_stream(self, stream_id: int) -> None:
-        logger.trace("close_stream stream_id=%r", stream_id)
-        del self.streams[stream_id]
-        del self.events[stream_id]
+        try:
+            logger.trace("close_stream stream_id=%r", stream_id)
+            del self.streams[stream_id]
+            del self.events[stream_id]
 
-        if not self.streams:
-            if self.state == ConnectionState.ACTIVE:
-                self.state = ConnectionState.IDLE
-            elif self.state == ConnectionState.FULL:
-                await self.aclose()
+            if not self.streams:
+                if self.state == ConnectionState.ACTIVE:
+                    self.state = ConnectionState.IDLE
+                elif self.state == ConnectionState.FULL:
+                    await self.aclose()
+        finally:
+            self.max_streams_semaphore.release()
 
 
 class AsyncHTTP2Stream:
@@ -256,7 +283,7 @@ class AsyncHTTP2Stream:
         timeout: TimeoutDict = None,
     ) -> Tuple[bytes, int, bytes, List[Tuple[bytes, bytes]], AsyncByteStream]:
         headers = [] if headers is None else [(k.lower(), v) for (k, v) in headers]
-        stream = AsyncByteStream() if stream is None else stream
+        stream = PlainByteStream(b"") if stream is None else stream
         timeout = {} if timeout is None else timeout
 
         # Send the request.
@@ -272,11 +299,11 @@ class AsyncHTTP2Stream:
         # Receive the response.
         status_code, headers = await self.receive_response(timeout)
         reason_phrase = get_reason_phrase(status_code)
-        stream = AsyncByteStream(
+        response_stream = AsyncIteratorByteStream(
             aiterator=self.body_iter(timeout), aclose_func=self._response_closed
         )
 
-        return (b"HTTP/2", status_code, reason_phrase, headers, stream)
+        return (b"HTTP/2", status_code, reason_phrase, headers, response_stream)
 
     async def send_headers(
         self,
@@ -288,7 +315,10 @@ class AsyncHTTP2Stream:
     ) -> None:
         scheme, hostname, port, path = url
         default_port = {b"http": 80, b"https": 443}.get(scheme)
-        authority = b"%s:%d" % (hostname, port) if port != default_port else hostname
+        if port is None or port == default_port:
+            authority = hostname
+        else:
+            authority = b"%s:%d" % (hostname, port)
 
         headers = [
             (b":method", method),
