@@ -1,5 +1,5 @@
 from ssl import SSLContext
-from typing import List, Optional, Tuple, Union
+from typing import List, Optional, Tuple
 
 from .._backends.auto import SyncLock, SyncSocketStream, SyncBackend
 from .._types import URL, Headers, Origin, TimeoutDict
@@ -10,8 +10,7 @@ from .base import (
     ConnectionState,
     NewConnectionRequired,
 )
-from .http2 import SyncHTTP2Connection
-from .http11 import SyncHTTP11Connection
+from .http import SyncBaseHTTPConnection
 
 logger = get_logger(__name__)
 
@@ -23,22 +22,39 @@ class SyncHTTPConnection(SyncHTTPTransport):
         http2: bool = False,
         ssl_context: SSLContext = None,
         socket: SyncSocketStream = None,
+        local_address: str = None,
     ):
         self.origin = origin
         self.http2 = http2
         self.ssl_context = SSLContext() if ssl_context is None else ssl_context
         self.socket = socket
+        self.local_address = local_address
 
         if self.http2:
             self.ssl_context.set_alpn_protocols(["http/1.1", "h2"])
 
-        self.connection: Union[None, SyncHTTP11Connection, SyncHTTP2Connection] = None
+        self.connection: Optional[SyncBaseHTTPConnection] = None
         self.is_http11 = False
         self.is_http2 = False
         self.connect_failed = False
         self.expires_at: Optional[float] = None
         self.backend = SyncBackend()
 
+    def __repr__(self) -> str:
+        http_version = "UNKNOWN"
+        if self.is_http11:
+            http_version = "HTTP/1.1"
+        elif self.is_http2:
+            http_version = "HTTP/2"
+        return f"<SyncHTTPConnection http_version={http_version} state={self.state}>"
+
+    def info(self) -> str:
+        if self.connection is None:
+            return "Not connected"
+        elif self.state == ConnectionState.PENDING:
+            return "Connecting"
+        return self.connection.info()
+
     @property
     def request_lock(self) -> SyncLock:
         # We do this lazily, to make sure backend autodetection always
@@ -83,7 +99,7 @@ class SyncHTTPConnection(SyncHTTPTransport):
         ssl_context = self.ssl_context if scheme == b"https" else None
         try:
             return self.backend.open_tcp_stream(
-                hostname, port, ssl_context, timeout
+                hostname, port, ssl_context, timeout, local_address=self.local_address
             )
         except Exception:
             self.connect_failed = True
@@ -95,11 +111,15 @@ class SyncHTTPConnection(SyncHTTPTransport):
             "create_connection socket=%r http_version=%r", socket, http_version
         )
         if http_version == "HTTP/2":
+            from .http2 import SyncHTTP2Connection
+
             self.is_http2 = True
             self.connection = SyncHTTP2Connection(
                 socket=socket, backend=self.backend, ssl_context=self.ssl_context
             )
         else:
+            from .http11 import SyncHTTP11Connection
+
             self.is_http11 = True
             self.connection = SyncHTTP11Connection(
                 socket=socket, ssl_context=self.ssl_context
@@ -111,7 +131,7 @@ class SyncHTTPConnection(SyncHTTPTransport):
             return ConnectionState.CLOSED
         elif self.connection is None:
             return ConnectionState.PENDING
-        return self.connection.state
+        return self.connection.get_state()
 
     def is_connection_dropped(self) -> bool:
         return self.connection is not None and self.connection.is_connection_dropped()
@@ -123,6 +143,10 @@ class SyncHTTPConnection(SyncHTTPTransport):
     def start_tls(self, hostname: bytes, timeout: TimeoutDict = None) -> None:
         if self.connection is not None:
             logger.trace("start_tls hostname=%r timeout=%r", hostname, timeout)
-            self.connection.start_tls(hostname, timeout)
+            self.socket = self.connection.start_tls(hostname, timeout)
             logger.trace("start_tls complete hostname=%r timeout=%r", hostname, timeout)
-            self.socket = self.connection.socket
+
+    def close(self) -> None:
+        with self.request_lock:
+            if self.connection is not None:
+                self.connection.close()
