@@ -1,11 +1,12 @@
 from ssl import SSLContext
-from typing import Iterator, Callable, Dict, Optional, Set, Tuple
+from typing import Iterator, Callable, Dict, List, Optional, Set, Tuple
+import warnings
 
 from .._backends.auto import SyncLock, SyncSemaphore, SyncBackend
-from .._exceptions import PoolTimeout
+from .._exceptions import PoolTimeout, LocalProtocolError, UnsupportedProtocol
 from .._threadlock import ThreadLock
 from .._types import URL, Headers, Origin, TimeoutDict
-from .._utils import get_logger, url_to_origin
+from .._utils import get_logger, origin_to_url_string, url_to_origin
 from .base import (
     SyncByteStream,
     SyncHTTPTransport,
@@ -71,31 +72,51 @@ class SyncConnectionPool(SyncHTTPTransport):
     verifying connections.
     * **max_connections** - `Optional[int]` - The maximum number of concurrent
     connections to allow.
-    * **max_keepalive** - `Optional[int]` - The maximum number of connections
-    to allow before closing keep-alive connections.
+    * **max_keepalive_connections** - `Optional[int]` - The maximum number of
+    connections to allow before closing keep-alive connections.
     * **keepalive_expiry** - `Optional[float]` - The maximum time to allow
     before closing a keep-alive connection.
     * **http2** - `bool` - Enable HTTP/2 support.
+    * **local_address** - `Optional[str]` - Local address to connect from.
     """
 
     def __init__(
         self,
         ssl_context: SSLContext = None,
         max_connections: int = None,
-        max_keepalive: int = None,
+        max_keepalive_connections: int = None,
         keepalive_expiry: float = None,
         http2: bool = False,
+        local_address: str = None,
+        max_keepalive: int = None,
     ):
+        if max_keepalive is not None:
+            warnings.warn(
+                "'max_keepalive' is deprecated. Use 'max_keepalive_connections'.",
+                DeprecationWarning,
+            )
+            max_keepalive_connections = max_keepalive
+
         self._ssl_context = SSLContext() if ssl_context is None else ssl_context
         self._max_connections = max_connections
         self._max_keepalive = max_keepalive
         self._keepalive_expiry = keepalive_expiry
         self._http2 = http2
+        self._local_address = local_address
         self._connections: Dict[Origin, Set[SyncHTTPConnection]] = {}
         self._thread_lock = ThreadLock()
         self._backend = SyncBackend()
         self._next_keepalive_check = 0.0
 
+        if http2:
+            try:
+                import h2
+            except ImportError:
+                raise ImportError(
+                    "Attempted to use http2=True, but the 'h2' "
+                    "package is not installed. Use 'pip install httpcore[http2]'."
+                )
+
     @property
     def _connection_semaphore(self) -> SyncSemaphore:
         # We do this lazily, to make sure backend autodetection always
@@ -124,11 +145,15 @@ class SyncConnectionPool(SyncHTTPTransport):
         stream: SyncByteStream = None,
         timeout: TimeoutDict = None,
     ) -> Tuple[bytes, int, bytes, Headers, SyncByteStream]:
-        assert url[0] in (b'http', b'https')
+        if url[0] not in (b"http", b"https"):
+            scheme = url[0].decode("latin-1")
+            raise UnsupportedProtocol(f"Unsupported URL protocol {scheme!r}")
+        if not url[1]:
+            raise LocalProtocolError("Missing hostname in URL.")
+
         origin = url_to_origin(url)
 
-        if self._keepalive_expiry is not None:
-            self._keepalive_sweep()
+        self._keepalive_sweep()
 
         connection: Optional[SyncHTTPConnection] = None
         while connection is None:
@@ -141,7 +166,10 @@ class SyncConnectionPool(SyncHTTPTransport):
 
                 if connection is None:
                     connection = SyncHTTPConnection(
-                        origin=origin, http2=self._http2, ssl_context=self._ssl_context,
+                        origin=origin,
+                        http2=self._http2,
+                        ssl_context=self._ssl_context,
+                        local_address=self._local_address,
                     )
                     logger.trace("created connection=%r", connection)
                     self._add_to_pool(connection, timeout=timeout)
@@ -179,6 +207,7 @@ class SyncConnectionPool(SyncHTTPTransport):
 
             if connection.state == ConnectionState.IDLE:
                 if connection.is_connection_dropped():
+                    logger.trace("removing dropped idle connection=%r", connection)
                     # IDLE connections that have been dropped should be
                     # removed from the pool.
                     connections_to_close.add(connection)
@@ -186,9 +215,11 @@ class SyncConnectionPool(SyncHTTPTransport):
                 else:
                     # IDLE connections that are still maintained may
                     # be reused.
+                    logger.trace("reusing idle http11 connection=%r", connection)
                     reuse_connection = connection
             elif connection.state == ConnectionState.ACTIVE and connection.is_http2:
                 # HTTP/2 connections may be reused.
+                logger.trace("reusing active http2 connection=%r", connection)
                 reuse_connection = connection
             elif connection.state == ConnectionState.PENDING:
                 # Pending connections may potentially be reused.
@@ -202,6 +233,7 @@ class SyncConnectionPool(SyncHTTPTransport):
         elif self._http2 and pending_connection is not None and not seen_http11:
             # If we have a PENDING connection, and no HTTP/1.1 connections
             # on this origin, then we can attempt to share the connection.
+            logger.trace("reusing pending connection=%r", connection)
             reuse_connection = pending_connection
 
         # Close any dropped connections.
@@ -238,13 +270,14 @@ class SyncConnectionPool(SyncHTTPTransport):
         """
         Remove any IDLE connections that have expired past their keep-alive time.
         """
-        assert self._keepalive_expiry is not None
+        if self._keepalive_expiry is None:
+            return
 
         now = self._backend.time()
         if now < self._next_keepalive_check:
             return
 
-        self._next_keepalive_check = now + 1.0
+        self._next_keepalive_check = now + min(1.0, self._keepalive_expiry)
         connections_to_close = set()
 
         for connection in self._get_all_connections():
@@ -264,12 +297,14 @@ class SyncConnectionPool(SyncHTTPTransport):
     ) -> None:
         timeout = {} if timeout is None else timeout
 
+        logger.trace("adding connection to pool=%r", connection)
         self._connection_semaphore.acquire(timeout=timeout.get("pool", None))
         with self._thread_lock:
             self._connections.setdefault(connection.origin, set())
             self._connections[connection.origin].add(connection)
 
     def _remove_from_pool(self, connection: SyncHTTPConnection) -> None:
+        logger.trace("removing connection from pool=%r", connection)
         with self._thread_lock:
             if connection in self._connections.get(connection.origin, set()):
                 self._connection_semaphore.release()
@@ -294,3 +329,16 @@ class SyncConnectionPool(SyncHTTPTransport):
         # Close all connections
         for connection in connections:
             connection.close()
+
+    def get_connection_info(self) -> Dict[str, List[str]]:
+        """
+        Returns a dict of origin URLs to a list of summary strings for each connection.
+        """
+        self._keepalive_sweep()
+
+        stats = {}
+        for origin, connections in self._connections.items():
+            stats[origin_to_url_string(origin)] = [
+                connection.info() for connection in connections
+            ]
+        return stats
