@@ -2,10 +2,10 @@
 Type definitions for type checking purposes.
 """
 
-from typing import Dict, List, Optional, Tuple, Union
+from typing import List, Mapping, Optional, Tuple, Union
 
 StrOrBytes = Union[str, bytes]
 Origin = Tuple[bytes, bytes, int]
-URL = Tuple[bytes, bytes, int, bytes]
+URL = Tuple[bytes, bytes, Optional[int], bytes]
 Headers = List[Tuple[bytes, bytes]]
-TimeoutDict = Dict[str, Optional[float]]
+TimeoutDict = Mapping[str, Optional[float]]
